package main

dynamodb_tables[r] {
    r := input.resource_changes[_]
    r.type == "aws_dynamodb_table"
}

################################################################################
## Test Cases                                                                 ##
################################################################################

# AWS DynamoDB tables are automatically encrypted at rest with an AWS owned Customer Master Key if an argument isn't specified.

# === Rule to deny server-side encryption if using AWS owned CMK ===
# TODO: Combine this and "db_server_side_encryption_aws_managed_cmk" with if-else
db_server_side_encryption_aws_owned_cmk[msg] {
    r := dynamodb_tables[_]
    # If enabled is false then server-side encryption is set to AWS owned CMK (shown as DEFAULT in the AWS console).
    r.change.after.server_side_encryption[_].enabled == false
    msg := sprintf(
        "%s: Server Side Encryption set to %q will use an AWS owned CMK. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address, r.change.after.server_side_encryption[_].enabled]
    )
}

deny_db_server_side_encryption_aws_owned_cmk[msg]{
  not array_contains(warning_mode, "db_server_side_encryption_aws_owned_cmk")
  db_server_side_encryption_aws_owned_cmk[_] != []
  msg := db_server_side_encryption_aws_owned_cmk[_]
}

warn_db_server_side_encryption_aws_owned_cmk[msg]{
  array_contains(warning_mode, "db_server_side_encryption_aws_owned_cmk")
  db_server_side_encryption_aws_owned_cmk[_] != []
  msg := db_server_side_encryption_aws_owned_cmk[_]
}

# === Rule to require Point in Time Recovery ===
db_point_in_time_recovery[msg] {
    r := dynamodb_tables[_]
    r.change.after.point_in_time_recovery[_].enabled == false
    msg := sprintf(
        "%s: Backups not enabled. Point in Time Recovery is %q. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address, r.change.after.point_in_time_recovery[_].enabled]
    )
}

deny_db_point_in_time_recovery[msg]{
  not array_contains(warning_mode, "db_point_in_time_recovery")
  db_point_in_time_recovery[_] != []
  msg := db_point_in_time_recovery[_]
}

warn_db_point_in_time_recovery[msg]{
  array_contains(warning_mode, "db_point_in_time_recovery")
  db_point_in_time_recovery[_] != []
  msg := db_point_in_time_recovery[_]
}