package main

elasticsearch_domains[r] {
    r := input.resource_changes[_]
    r.type == "aws_elasticsearch_domain"
}

################################################################################
## Test Cases                                                                 ##
################################################################################

# === Rule to require encryption at rest ===
# Case where encrypt_at_rest{} is not set in terraform
es_encryption_at_rest[reason] {
    r := elasticsearch_domains[_]
    encryption := object.get(r.change.after,"encrypt_at_rest",false)
    encryption == false
    reason := sprintf(
        "%s : Elasticsearch must have encryption at rest enabled. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address]
    )
}

deny_es_encryption_at_rest[msg]{
  not array_contains(warning_mode, "es_encryption_at_rest")
  es_encryption_at_rest[_] != []
  msg := es_encryption_at_rest[_]
}

warn_es_encryption_at_rest[msg]{
  array_contains(warning_mode, "es_encryption_at_rest")
  es_encryption_at_rest[_] != []
  msg := es_encryption_at_rest[_]
}

# === Rule to require encryption at rest ===
# Case where encryption_at_rest{} is explicitly set to false in terraform
es_encryption_at_rest_false[reason] {
    r := elasticsearch_domains[_]
    encryption := object.get(r.change.after.encrypt_at_rest[0],"enabled",false)
    encryption == false
    reason := sprintf(
        "%s : Elasticsearch must have encryption at rest enabled. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address]
    )
}

deny_es_encryption_at_rest_false[msg]{
  not array_contains(warning_mode, "es_encryption_at_rest_false")
  es_encryption_at_rest_false[_] != []
  msg := es_encryption_at_rest_false[_]
}

warn_es_encryption_at_rest_false[msg]{
  array_contains(warning_mode, "es_encryption_at_rest_false")
  es_encryption_at_rest_false[_] != []
  msg := es_encryption_at_rest_false[_]
}

# === Rule to require an instance type that supports encryption at rest ===
es_encryption_at_rest_unsupported[reason] {
    r := elasticsearch_domains[_]
    denied_es_instance_types := ["m3", "r3", "t2"] # These instance types don't support encryption at rest
    instance_type := r.change.after.cluster_config[_].instance_type
    startswith(instance_type, denied_es_instance_types[_])
    reason := sprintf(
        "%s : Elasticsearch instance '%s' doesn't support encryption at rest. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address, instance_type]
    )
}

deny_es_encryption_at_rest_unsupported[msg]{
  not array_contains(warning_mode, "es_encryption_at_rest_unsupported")
  es_encryption_at_rest_unsupported[_] != []
  msg := es_encryption_at_rest_unsupported[_]
}

warn_es_encryption_at_rest_unsupported[msg]{
  array_contains(warning_mode, "es_encryption_at_rest_unsupported")
  es_encryption_at_rest_unsupported[_] != []
  msg := es_encryption_at_rest_unsupported[_]
}

# Rule to require ES to reside within a VPC
es_without_vpc[reason] {
    r := elasticsearch_domains[_]
    vpc_conf := r.change.after.vpc_options
    vpc_conf == []
    reason := sprintf(
        "%s: Elasticsearch must reside within a VPC. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address]
    ) 
}

deny_es_without_vpc[msg]{
  not array_contains(warning_mode, "es_without_vpc")
  es_without_vpc[_] != []
  msg := es_without_vpc[_]
}

warn_es_without_vpc[msg]{
  array_contains(warning_mode, "es_without_vpc")
  es_without_vpc[_] != []
  msg := es_without_vpc[_]
}

# Rule to prevent node_to_node encryption from defaulting to false
es_without_ntn_encryption[reason] {
    r := elasticsearch_domains[_]
    ntn_encryption := object.get(r.change.after,"node_to_node_encryption",false)
    ntn_encryption == false
    reason := sprintf(
        "%s: Elasticsearch must have node_to_node encryption enabled. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address]
    )
}

deny_es_without_ntn_encryption[msg]{
  not array_contains(warning_mode, "es_without_ntn_encryption")
  es_without_ntn_encryption[_] != []
  msg := es_without_ntn_encryption[_]
}

warn_es_without_ntn_encryption[msg]{
  array_contains(warning_mode, "es_without_ntn_encryption")
  es_without_ntn_encryption[_] != []
  msg := es_without_ntn_encryption[_]
}

# Rule to prevent node_to_node encryption from being configured to false
es_with_ntn_encryption_false[reason] {
	r := elasticsearch_domains[_]
    ntn_encryption := object.get(r.change.after.node_to_node_encryption[0],"enabled",false)
    ntn_encryption == false
    reason := sprintf(
        "%s: Elasticsearch node_to_node encryption must be set as enabled. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address]
    ) 
}

deny_es_with_ntn_encryption_false[msg]{
  not array_contains(warning_mode, "es_with_ntn_encryption_false")
  es_with_ntn_encryption_false[_] != []
  msg := es_with_ntn_encryption_false[_]
}

warn_es_with_ntn_encryption_false[msg]{
  array_contains(warning_mode, "es_with_ntn_encryption_false")
  es_with_ntn_encryption_false[_] != []
  msg := es_with_ntn_encryption_false[_]
}