package main

# Test elasticsearch for encryption at rest
test_es_encryption_at_rest_invalid {
    i := data.mock.invalid_es_input
    r1 := warn_es_encryption_at_rest with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_es_encryption_at_rest with input as i with data.config.warn_mode as ["es_encryption_at_rest"]
    count(r2) == 1
    r3 := deny_es_encryption_at_rest with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_es_encryption_at_rest with input as i with data.config.warn_mode as ["es_encryption_at_rest"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

test_es_encryption_at_rest_valid {
    i := data.mock.valid_es_input
    r1 := warn_es_encryption_at_rest with input as i with data.config.warn_mode as []
    r2 := warn_es_encryption_at_rest with input as i with data.config.warn_mode as ["es_encryption_at_rest"]
    r3 := deny_es_encryption_at_rest with input as i with data.config.warn_mode as []
    r4 := deny_es_encryption_at_rest with input as i with data.config.warn_mode as ["es_encryption_at_rest"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}

# Test elasticsearch input without encryption at rest, and with encryption at rest set to false
test_es_encryption_at_rest_false_invalid {
    i := data.mock.invalid_es_input
    r1 := warn_es_encryption_at_rest_false with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_es_encryption_at_rest_false with input as i with data.config.warn_mode as ["es_encryption_at_rest_false"]
    count(r2) == 1
    r3 := deny_es_encryption_at_rest_false with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_es_encryption_at_rest_false with input as i with data.config.warn_mode as ["es_encryption_at_rest_false"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

# Test elasticsearch input without encryption at rest on unsupported instances
test_es_encryption_at_rest_unsupported_invalid {
    i := data.mock.invalid_es_instances
    r1 := warn_es_encryption_at_rest_unsupported with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_es_encryption_at_rest_unsupported with input as i with data.config.warn_mode as ["es_encryption_at_rest_unsupported"]
    count(r2) == 2
    r3 := deny_es_encryption_at_rest_unsupported with input as i with data.config.warn_mode as []
    count(r3) == 2
    r4 := deny_es_encryption_at_rest_unsupported with input as i with data.config.warn_mode as ["es_encryption_at_rest_unsupported"]
    count(r4) == 0
    count(r1) + count(r2) == 2 # Warning mode is off
    count(r3) + count(r4) == 2 # Warning mode is on
}

# Test ES for valid VPC
test_es_without_vpc_invalid {
    i := data.mock.invalid_es_vpc
    r1 := warn_es_without_vpc with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_es_without_vpc with input as i with data.config.warn_mode as ["es_without_vpc"]
    count(r2) == 1
    r3 := deny_es_without_vpc with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_es_without_vpc with input as i with data.config.warn_mode as ["es_without_vpc"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

test_es_without_vpc_valid {
    i := data.mock.valid_es_vpc
    r1 := warn_es_without_vpc with input as i with data.config.warn_mode as []
    r2 := warn_es_without_vpc with input as i with data.config.warn_mode as ["es_without_vpc"]
    r3 := deny_es_without_vpc with input as i with data.config.warn_mode as []
    r4 := deny_es_without_vpc with input as i with data.config.warn_mode as ["es_without_vpc"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}

#
test_es_without_ntn_encryption_invalid {
    i := data.mock.invalid_es_with_ntn_encryption
    r1 := warn_es_without_ntn_encryption with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_es_without_ntn_encryption with input as i with data.config.warn_mode as ["es_without_ntn_encryption"]
    count(r2) == 1
    r3 := deny_es_without_ntn_encryption with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_es_without_ntn_encryption with input as i with data.config.warn_mode as ["es_without_ntn_encryption"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

test_es_without_ntn_encryption_valid {
    i := data.mock.valid_es_with_ntn_encryption
    r1 := warn_es_without_ntn_encryption with input as i with data.config.warn_mode as []
    r2 := warn_es_without_ntn_encryption with input as i with data.config.warn_mode as ["es_without_ntn_encryption"]
    r3 := deny_es_without_ntn_encryption with input as i with data.config.warn_mode as []
    r4 := deny_es_without_ntn_encryption with input as i with data.config.warn_mode as ["es_without_ntn_encryption"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}

test_es_with_ntn_encryption_false_invalid {
    i := data.mock.invalid_es_with_ntn_encryption
    r1 := warn_es_with_ntn_encryption_false with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_es_with_ntn_encryption_false with input as i with data.config.warn_mode as ["es_with_ntn_encryption_false"]
    count(r2) == 0
    r3 := deny_es_with_ntn_encryption_false with input as i with data.config.warn_mode as []
    count(r3) == 0
    r4 := deny_es_with_ntn_encryption_false with input as i with data.config.warn_mode as ["es_with_ntn_encryption_false"]
    count(r4) == 0
    count(r1) + count(r2) == 0 # Warning mode is off
    count(r3) + count(r4) == 0 # Warning mode is on
}

test_es_with_ntn_encryption_false_valid {
    i := data.mock.valid_es_with_ntn_encryption
    r1 := warn_es_with_ntn_encryption_false with input as i with data.config.warn_mode as []
    r2 := warn_es_with_ntn_encryption_false with input as i with data.config.warn_mode as ["es_with_ntn_encryption_false"]
    r3 := deny_es_with_ntn_encryption_false with input as i with data.config.warn_mode as []
    r4 := deny_es_with_ntn_encryption_false with input as i with data.config.warn_mode as ["es_with_ntn_encryption_false"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}