package main

not_allowed_arns = [
  "arn:aws:iam::aws:policy/AdministratorAccess",
  "arn:aws:iam::aws:policy/AmazonDynamoDBFullAccess",
  "arn:aws:iam::aws:policy/AmazonRDSFullAccess",
  "arn:aws:iam::aws:policy/IAMFullAccess"
  ]
denied_policy_permissions = [
  "rds:*",
  "dynamodb:*",
  "dynamodb:Restore*",
  "dynamodb:Update*",
  "dynamodb:RestoreTableFromBackup",
  "dynamodb:RestoreTableToPointInTime",
  "dynamodb:UpdateGlobalTable",
  "dynamodb:UpdateGlobalTableSettings",
  "dynamodb:UpdateTable",
  "dynamodb:UpdateTableReplicaAutoScaling",
  "rds:Delete*",
  "rds:Modify*",
  "rds:Restore*",
  "rds:Remove*",
  "rds:CancelExportTask",
  "rds:DeleteCustomAvailabilityZone",
  "rds:DeleteDBCluster",
  "rds:DeleteDBClusterEndpoint",
  "rds:DeleteDBClusterParameterGroup",
  "rds:DeleteDBClusterSnapshot",
  "rds:DeleteDBInstance",
  "rds:DeleteDBInstanceAutomatedBackup",
  "rds:DeleteDBParameterGroup",
  "rds:DeleteDBProxy",
  "rds:DeleteDBProxyEndpoint",
  "rds:DeleteDBSecurityGroup",
  "rds:DeleteDBSnapshot",
  "rds:DeleteDBSubnetGroup",
  "rds:DeleteEventSubscription",
  "rds:DeleteGlobalCluster",
  "rds:DeleteInstallationMedia",
  "rds:DeleteOptionGroup",
  "rds:DeregisterDBProxyTargets",
  "rds:ModifyCertificates",
  "rds:ModifyCurrentDBClusterCapacity",
  "rds:ModifyDBCluster",
  "rds:ModifyDBClusterEndpoint",
  "rds:ModifyDBClusterParameterGroup",
  "rds:ModifyDBClusterSnapshotAttribute",
  "rds:ModifyDBInstance",
  "rds:ModifyDBParameterGroup",
  "rds:ModifyDBProxy",
  "rds:ModifyDBProxyEndpoint",
  "rds:ModifyDBProxyTargetGroup",
  "rds:ModifyDBSnapshot",
  "rds:ModifyDBSnapshotAttribute",
  "rds:ModifyDBSubnetGroup",
  "rds:ModifyEventSubscription",
  "rds:ModifyGlobalCluster",
  "rds:ModifyOptionGroup",
  "rds:PromoteReadReplica",
  "rds:PromoteReadReplicaDBCluster",
  "rds:RemoveFromGlobalCluster",
  "rds:RemoveRoleFromDBCluster",
  "rds:RemoveRoleFromDBInstance",
  "rds:RestoreDBClusterFromS3",
  "rds:RestoreDBClusterFromSnapshot",
  "rds:RestoreDBClusterToPointInTime",
  "rds:RestoreDBInstanceFromDBSnapshot",
  "rds:RestoreDBInstanceFromS3",
  "rds:RestoreDBInstanceToPointInTime",
  "rds:RevokeDBSecurityGroupIngress",
  "glue:UpdateDevEndpoint",
  "glue:CreateDevEndpoint",
  "glue:*",
  "iam:*",
  "iam:PassRole"
  ]

iam_users[r] {
    r := input.resource_changes[_]
    r.type == "aws_iam_user"
}

iam_policies[r] {
    r := input.resource_changes[_]
    r.type == "aws_iam_policy"
}

iam_roles[r] {
    r := input.resource_changes[_]
    r.type == "aws_iam_role"
}

iam_attachments[r] {
    r := input.resource_changes[_]
    r.type == "aws_iam_role_policy_attachment"
}

aws_iam_user_login_profile[r] {
    r := input.resource_changes[_]
    r.type == "aws_iam_user_login_profile"
}

#Filter for when permissions are defined as an array "Action": ["esc2:*","rds:*"]
policy_filter(unmarshaled) {
	unmarshaled["Effect"] == "Allow"
    array_contains(denied_policy_permissions,unmarshaled["Action"]) == true
    not unmarshaled["Condition"]
}

#Filter for when permissions are defined as a single key value :"Action": "esc2:*"
policy_filter(unmarshaled) {
	unmarshaled["Effect"] == "Allow"
    array_contains(denied_policy_permissions,unmarshaled["Action"][_]) == true
    not unmarshaled["Condition"]
}

allow_policy(r) {
    js_unmarshal = json.unmarshal(r)
    policy_filter(js_unmarshal["Statement"][_])
}

################################################################################
## Test Cases                                                                 ##
################################################################################

# === Check for IAM policy attachment ===
iam_policy_attachments[reason] {
    r := iam_attachments[_]
    array_contains(not_allowed_arns, r.change.after.policy_arn)
    reason := sprintf(
        "%s: Policy attachment %q is not allowed. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address, r.change.after.policy_arn]
    )
}

deny_iam_policy_attachments[msg]{
  not array_contains(warning_mode, "iam_policy_attachments")
  iam_policy_attachments[_] != []
  msg := iam_policy_attachments[_]
}

warn_iam_policy_attachments[msg]{
  array_contains(warning_mode, "iam_policy_attachments")
  iam_policy_attachments[_] != []
  msg := iam_policy_attachments[_]
}

# === Check for elevated IAM permissions ===
iam_policy_permissions[reason] {
	r := iam_policies[_]
	allow_policy(r.change.after.policy)
    reason := sprintf(
        "%s: Policy permissions are elevated. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address]
    )
}

deny_iam_policy_permissions[msg]{
  not array_contains(warning_mode, "iam_policy_permissions")
  iam_policy_permissions[_] != []
  msg := iam_policy_permissions[_]
}

warn_iam_policy_permissions[msg]{
  array_contains(warning_mode, "iam_policy_permissions")
  iam_policy_permissions[_] != []
  msg := iam_policy_permissions[_]
}

# === Check for inline policy with elevated permissions ===
iam_inline_policy_permissions[reason] {
    r := iam_roles[_]
    allow_policy(r.change.after.inline_policy[_]["policy"])
    reason := sprintf(
            "%s: IAM role has inline policy with elevated permissions. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
            [r.address]
        )
}

deny_iam_inline_policy_permissions[msg]{
  not array_contains(warning_mode, "iam_inline_policy_permissions")
  iam_inline_policy_permissions[_] != []
  msg := iam_inline_policy_permissions[_]
}

warn_iam_inline_policy_permissions[msg]{
  array_contains(warning_mode, "iam_inline_policy_permissions")
  iam_inline_policy_permissions[_] != []
  msg := iam_inline_policy_permissions[_]
}

# === Check for creation of IAM users ===
iam_console_user_creation[reason]{
    r := aws_iam_user_login_profile[_]
    reason := sprintf(
        "%s: IAM users with console access are not allowed. See https://gitlab.bit9.local/production-infrastructure/cloud-governance/policy-packs/-/blob/master/policies/terraform/README.md for more details.",
        [r.address]
    )
}

deny_iam_console_user_creation[msg]{
  not array_contains(warning_mode, "iam_console_user_creation")
  iam_console_user_creation[_] != []
  msg := iam_console_user_creation[_]
}

warn_iam_console_user_creation[msg]{
  array_contains(warning_mode, "iam_console_user_creation")
  iam_console_user_creation[_] != []
  msg := iam_console_user_creation[_]
}