package main

# Test plan for creation of IAM users
test_iam_console_user_creation_invalid {
    i := data.mock.deny_iam_console_user
    r1 := warn_iam_console_user_creation with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_iam_console_user_creation with input as i with data.config.warn_mode as ["iam_console_user_creation"]
    count(r2) == 1
    r3 := deny_iam_console_user_creation with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_iam_console_user_creation with input as i with data.config.warn_mode as ["iam_console_user_creation"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

#
test_iam_inline_policy_permissions_invalid {
    i := data.mock.invalid_iam_role_inline
    r1 := warn_iam_inline_policy_permissions with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_iam_inline_policy_permissions with input as i with data.config.warn_mode as ["iam_inline_policy_permissions"]
    count(r2) == 1
    r3 := deny_iam_inline_policy_permissions with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_iam_inline_policy_permissions with input as i with data.config.warn_mode as ["iam_inline_policy_permissions"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

test_iam_inline_policy_permissions_valid {
    i := data.mock.valid_iam_role_inline
    r1 := warn_iam_inline_policy_permissions with input as i with data.config.warn_mode as []
    r2 := warn_iam_inline_policy_permissions with input as i with data.config.warn_mode as ["iam_inline_policy_permissions"]
    r3 := deny_iam_inline_policy_permissions with input as i with data.config.warn_mode as []
    r4 := deny_iam_inline_policy_permissions with input as i with data.config.warn_mode as ["iam_inline_policy_permissions"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}

#
test_iam_policy_permissions_invalid {
    i := data.mock.invalid_iam_policy
    r1 := warn_iam_policy_permissions with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_iam_policy_permissions with input as i with data.config.warn_mode as ["iam_policy_permissions"]
    count(r2) == 1
    r3 := deny_iam_policy_permissions with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_iam_policy_permissions with input as i with data.config.warn_mode as ["iam_policy_permissions"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

test_iam_policy_permissions_valid {
    i := data.mock.valid_iam_policy
    r1 := warn_iam_policy_permissions with input as i with data.config.warn_mode as []
    r2 := warn_iam_policy_permissions with input as i with data.config.warn_mode as ["iam_policy_permissions"]
    r3 := deny_iam_policy_permissions with input as i with data.config.warn_mode as []
    r4 := deny_iam_policy_permissions with input as i with data.config.warn_mode as ["iam_policy_permissions"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}

# Test for IAM policy attachments
test_iam_policy_attachments_invalid {
    i := data.mock.invalid_iam_attachement
    r1 := warn_iam_policy_attachments with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_iam_policy_attachments with input as i with data.config.warn_mode as ["iam_policy_attachments"]
    count(r2) == 1
    r3 := deny_iam_policy_attachments with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_iam_policy_attachments with input as i with data.config.warn_mode as ["iam_policy_attachments"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

# Test for IAM policy attachments with valid plan
test_iam_policy_attachments_valid {
    i := data.mock.valid_iam_attachement
    r1 := warn_iam_policy_attachments with input as i with data.config.warn_mode as []
    r2 := warn_iam_policy_attachments with input as i with data.config.warn_mode as ["iam_policy_attachments"]
    r3 := deny_iam_policy_attachments with input as i with data.config.warn_mode as []
    r4 := deny_iam_policy_attachments with input as i with data.config.warn_mode as ["iam_policy_attachments"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}

test_allow_policy {
    json_string_invalid = "{\"Statement\":[{\"Action\":\"dynamodb:*\",\"Effect\":\"Allow\",\"Resource\":\"*\"},{\"Action\":[\"ec2:*\",\"s3:*\"],\"Effect\":\"Allow\",\"Resource\":\"*\"}],\"Version\":\"2012-10-17\"}"
	json_string_valid = "{\"Statement\":[{\"Action\":\"dynamodb:*\",\"Effect\":\"Deny\",\"Resource\":\"*\"},{\"Action\":[\"ec2:*\",\"s3:*\"],\"Effect\":\"Allow\",\"Resource\":\"*\"}],\"Version\":\"2012-10-17\"}"
	allow_policy(json_string_invalid)
	not allow_policy(json_string_valid)
}

# Additional test cases to verify iam:* and iam:PassRole are correctly configured
test_iam_policy_passrole_invalid {
    i := data.mock.invalid_iam_passrole_policy
    r1 := warn_iam_policy_permissions with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_iam_policy_permissions with input as i with data.config.warn_mode as ["iam_policy_permissions"]
    count(r2) == 1
    r3 := deny_iam_policy_permissions with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_iam_policy_permissions with input as i with data.config.warn_mode as ["iam_policy_permissions"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

test_iam_policy_passrole_valid {
    i := data.mock.valid_iam_passrole_policy
    r1 := warn_iam_policy_permissions with input as i with data.config.warn_mode as []
    r2 := warn_iam_policy_permissions with input as i with data.config.warn_mode as ["iam_policy_permissions"]
    r3 := deny_iam_policy_permissions with input as i with data.config.warn_mode as []
    r4 := deny_iam_policy_permissions with input as i with data.config.warn_mode as ["iam_policy_permissions"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}