package kubernetes

# Containers for Jobs/Pods/Deployments should not be using :latest image tags
# Pod Version, Untagged
resource_container_using_latest[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not regex.match(`:([\w.\-_]{1,127})$`, containers.image)
    reason := sprintf("%s %s: Container %s image %s is untagged. Using an explicit non-latest tag is required to avoid accidental upgrades", [r.kind, r.metadata.name, containers.name, containers.image])
}

# Pod Version, :latest
resource_container_using_latest[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    regex.match(`.*\:latest$`, containers.image)
    reason := sprintf("%s %s: Container %s image %s is using :latest tag. Using an explicit non-latest tag is required to avoid accidental upgrades", [r.kind, r.metadata.name, containers.name, containers.image])
}

# Deployments & Jobs Version, Untagged
resource_container_using_latest[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    not regex.match(`:([\w.\-_]{1,127})$`, containers.image)
    reason := sprintf("%s %s: Container %s image %s is untagged. Using an explicit non-latest tag is required to avoid accidental upgrades", [r.kind, r.metadata.name, containers.name, containers.image])
}

# Deployments & Jobs Version, :latest
resource_container_using_latest[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    regex.match(`.*\:latest$`, containers.image)
    reason := sprintf("%s %s: Container %s image %s is using :latest tag. Using an explicit non-latest tag is required to avoid accidental upgrades", [r.kind, r.metadata.name, containers.name, containers.image])
}

# Actual Test Cases
deny_resource_container_using_latest[msg]{
  not array_contains(warning_mode, "resource_container_using_latest")
  resource_container_using_latest[_] != []
  msg := resource_container_using_latest[_]
}

warn_resource_container_using_latest[msg]{
  array_contains(warning_mode, "resource_container_using_latest")
  resource_container_using_latest[_] != []
  msg := resource_container_using_latest[_]
}