package kubernetes

# Containers for Jobs/Pods/Deployments should have containers[_].securityContext.readOnlyRootFilesystem set to True
# Pod Version
resource_container_readonlyrootfilesystem[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not containers.securityContext.readOnlyRootFilesystem
    reason := sprintf("%s %s: Container %s does not have a readOnlyRootFilesystem. Set securityContext.readOnlyRootFilesystem", [r.kind, r.metadata.name, containers.name])
}

# Deployments Version
resource_container_readonlyrootfilesystem[reason] {
    r := get_containers[_]
    r.kind == "Deployment"
    containers := r.spec.template.spec.containers[_]
    not containers.securityContext.readOnlyRootFilesystem
    reason := sprintf("%s %s: Container %s does not have a readOnlyRootFilesystem. Set securityContext.readOnlyRootFilesystem", [r.kind, r.metadata.name, containers.name])
}

# Jobs Version
resource_container_readonlyrootfilesystem[reason] {
    r := get_containers[_]
    r.kind == "Job"
    infra_chart_exceptions(r.metadata.name) # exception function for infra-charts, see k8s_main.rego
    containers := r.spec.template.spec.containers[_]
    not containers.securityContext.readOnlyRootFilesystem
    reason := sprintf("%s %s: Container %s does not have a readOnlyRootFilesystem. Set securityContext.readOnlyRootFilesystem", [r.kind, r.metadata.name, containers.name])
}

# Actual Test Cases
deny_resource_container_readonlyrootfilesystem[msg]{
  not array_contains(warning_mode, "resource_container_readonlyrootfilesystem")
  findings := resource_container_readonlyrootfilesystem
  findings[_] != []
  msg := findings[_]
}

warn_resource_container_readonlyrootfilesystem[msg]{
  array_contains(warning_mode, "resource_container_readonlyrootfilesystem")
  findings := resource_container_readonlyrootfilesystem
  findings[_] != []
  msg := findings[_]
}

