package kubernetes

# Containers for Jobs/Pods/Deployments should have containers[_].resources.requests configured for CPU & Memory
# Pod CPU Version
resource_missing_container_resources_requests[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not containers.resources.requests.cpu
    reason := sprintf("%s %s: Container %s is missing container CPU resource request configuration. Set resources.requests.cpu", [r.kind, r.metadata.name, containers.name])
}

# Pod Memory Version
resource_missing_container_resources_requests[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not containers.resources.requests.memory
    reason := sprintf("%s %s: Container %s is missing container Memory resource request configuration. Set resources.requests.memory", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs CPU Version
resource_missing_container_resources_requests[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.requests.cpu
    reason := sprintf("%s %s: Container %s is missing container CPU resource request configuration. Set resources.requests.cpu", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs Memory Version
resource_missing_container_resources_requests[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.requests.memory
    reason := sprintf("%s %s: Container %s is missing container Memory resource request configuration. Set resources.requests.memory", [r.kind, r.metadata.name, containers.name])
}

# Actual Test Cases
deny_resource_missing_container_resources_requests[msg]{
  not array_contains(warning_mode, "resource_missing_container_resources_requests")
  resource_missing_container_resources_requests[_] != []
  msg := resource_missing_container_resources_requests[_]
}

warn_resource_missing_container_resources_requests[msg]{
  array_contains(warning_mode, "resource_missing_container_resources_requests")
  resource_missing_container_resources_requests[_] != []
  msg := resource_missing_container_resources_requests[_]
}

# Containers for Jobs/Pods/Deployments should have containers[_].resources.limits configured for CPU & Memory
# Pod CPU Version
resource_missing_container_resources_limits[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not containers.resources.limits.cpu
    reason := sprintf("%s %s: Container %s is missing container CPU resource request configuration. Set resources.limits.cpu", [r.kind, r.metadata.name, containers.name])
}

# Pod Memory Version
resource_missing_container_resources_limits[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not containers.resources.limits.memory
    reason := sprintf("%s %s: Container %s is missing container Memory resource request configuration. Set resources.limits.memory", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs CPU Version
resource_missing_container_resources_limits[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.limits.cpu
    reason := sprintf("%s %s: Container %s is missing container CPU resource request configuration. Set resources.limits.cpu", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs Memory Version
resource_missing_container_resources_limits[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.limits.memory
    reason := sprintf("%s %s: Container %s is missing container Memory resource request configuration. Set resources.limits.memory", [r.kind, r.metadata.name, containers.name])
}

# Actual Test Cases
deny_resource_missing_container_resources_limits[msg]{
  not array_contains(warning_mode, "resource_missing_container_resources_limits")
  resource_missing_container_resources_limits[_] != []
  msg := resource_missing_container_resources_limits[_]
}

warn_resource_missing_container_resources_limits[msg]{
  array_contains(warning_mode, "resource_missing_container_resources_limits")
  resource_missing_container_resources_limits[_] != []
  msg := resource_missing_container_resources_limits[_]
}

# Containers for Jobs/Pods/Deployments should have containers[_].resources.limits set equal to containers[_].resources.requests
# Pod CPU Version
# NOTE: These are p3 and will only run in warning mode
warn_resource_container_req_eq_limit[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not containers.resources.limits.cpu == containers.resources.requests.cpu
    reason := sprintf("[Recommended] %s %s: Container %s has different requests (%s) than limits (%s). Set resources.limits.cpu equal to resources.requests.cpu", [r.kind, r.metadata.name, containers.name, containers.resources.requests.cpu, containers.resources.limits.cpu])
}

# Deployments & Jobs CPU Version
# NOTE: These are p3 and will only run in warning mode
warn_resource_container_req_eq_limit[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.limits.cpu == containers.resources.requests.cpu
    reason := sprintf("[Recommended] %s %s: Container %s has different requests (%s) than limits (%s). Set resources.limits.cpu equal to resources.requests.cpu", [r.kind, r.metadata.name, containers.name, containers.resources.requests.cpu, containers.resources.limits.cpu])
}

# Pod CPU Version
# NOTE: These are p3 and will only run in warning mode
warn_resource_container_req_eq_limit[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not containers.resources.limits.memory == containers.resources.requests.memory
    reason := sprintf("[Recommended] %s %s: Container %s has different requests (%s) than limits (%s). Set resources.limits.memory equal to resources.requests.memory", [r.kind, r.metadata.name, containers.name, containers.resources.requests.memory, containers.resources.limits.memory])
}

# Deployments & Jobs CPU Version
# NOTE: These are p3 and will only run in warning mode
warn_resource_container_req_eq_limit[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.limits.memory == containers.resources.requests.memory
    reason := sprintf("[Recommended] %s %s: Container %s has different requests (%s) than limits (%s). Set resources.limits.memory equal to resources.requests.memory", [r.kind, r.metadata.name, containers.name, containers.resources.requests.memory, containers.resources.limits.memory])
}

# Pod ephemeral-storage Version
# NOTE: These are p3 and will only run in warning mode
warn_resource_container_req_eq_limit[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not containers.resources.limits["ephemeral-storage"] == containers.resources.requests["ephemeral-storage"]
    reason := sprintf("[Recommended] %s %s: Container %s has different requests (%s) than limits (%s). Set resources.limits.ephemeral-storage equal to resources.requests.ephemeral-storage", [r.kind, r.metadata.name, containers.name, containers.resources.requests["ephemeral-storage"], containers.resources.limits["ephemeral-storage"]])
}

# Deployments & Jobs ephemeral-storage Version
# NOTE: These are p3 and will only run in warning mode
warn_resource_container_req_eq_limit[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    not containers.resources.limits["ephemeral-storage"] == containers.resources.requests["ephemeral-storage"]
    reason := sprintf("[Recommended] %s %s: Container %s has different requests (%s) than limits (%s). Set resources.limits.ephemeral-storage equal to resources.requests.ephemeral-storage", [r.kind, r.metadata.name, containers.name, containers.resources.requests["ephemeral-storage"], containers.resources.limits["ephemeral-storage"]])
}