package kubernetes

# Containers for Jobs/Pods/Deployments should have containers[_].securityContext.runAsUser set to a valid UUID
# Pod Version - No UUID
resource_container_run_as_user[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not containers.securityContext.runAsUser
    reason := sprintf("%s %s: Container %s does not have a User ID set. Set securityContext.runAsUser", [r.kind, r.metadata.name, containers.name])
}

# Pod Version - Root UUID
resource_container_run_as_user[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    containers.securityContext.runAsUser == 0
    reason := sprintf("%s %s: Container %s has a User ID set to the Root user. Set securityContext.runAsUser != 0", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs Version - No UUID
resource_container_run_as_user[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    not containers.securityContext.runAsUser
    reason := sprintf("%s %s: Container %s does not have a User ID set. Set securityContext.runAsUser", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs Version - Root UUID
resource_container_run_as_user[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    containers.securityContext.runAsUser == 0
    reason := sprintf("%s %s: Container %s has a User ID set to the Root user. Set securityContext.runAsUser != 0", [r.kind, r.metadata.name, containers.name])
}

# Actual Test Cases
deny_resource_container_run_as_user[msg]{
  not array_contains(warning_mode, "resource_container_run_as_user")
  findings := resource_container_run_as_user
  findings[_] != []
  msg := findings[_]
}

warn_resource_container_run_as_user[msg]{
  array_contains(warning_mode, "resource_container_run_as_user")
  findings := resource_container_run_as_user
  findings[_] != []
  msg := findings[_]
}

# Containers for Jobs/Pods/Deployments should have containers[_].securityContext.runAsGroup set to a valid GUID
# Pod Version - No GUID
resource_container_run_as_group[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    not containers.securityContext.runAsGroup
    reason := sprintf("%s %s: Container %s does not have a Group ID set. Set securityContext.runAsGroup", [r.kind, r.metadata.name, containers.name])
}

# Pod Version - Root GUID
resource_container_run_as_group[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    containers.securityContext.runAsGroup == 0
    reason := sprintf("%s %s: Container %s has a Group ID set to the Root group. Set securityContext.runAsGroup != 0", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs Version - No GUID
resource_container_run_as_group[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    not containers.securityContext.runAsGroup
    reason := sprintf("%s %s: Container %s does not have a Group ID set. Set securityContext.runAsGroup", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs Version - Root GUID
resource_container_run_as_group[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    containers.securityContext.runAsGroup == 0
    reason := sprintf("%s %s: Container %s has a Group ID set to the Root group. Set securityContext.runAsGroup != 0", [r.kind, r.metadata.name, containers.name])
}

# Actual Test Cases
deny_resource_container_run_as_group[msg]{
  not array_contains(warning_mode, "resource_container_run_as_group")
  findings := resource_container_run_as_group
  findings[_] != []
  msg := findings[_]
}

warn_resource_container_run_as_group[msg]{
  array_contains(warning_mode, "resource_container_run_as_group")
  findings := resource_container_run_as_group
  findings[_] != []
  msg := findings[_]
}