package kubernetes

# Containers for Jobs/Pods/Deployments should have an unprivileged security context set
# Deployments & Jobs Version, Set in container level
resource_container_security_context_privileged[reason] {
    r := get_containers[_]
    containers := r.spec.template.spec.containers[_]
    containers.securityContext["privileged"] == true
    reason := sprintf("%s %s: Container %s securityContext.privileged should not be set to 'true'.", [r.kind, r.metadata.name, containers.name])
}

# Deployments & Jobs Version, Set in top level
resource_container_security_context_privileged[reason] {
    r := get_containers[_]
    r.spec.template.spec.securityContext["privileged"] == true
    reason := sprintf("%s %s: securityContext.privileged should not be set to 'true'.", [r.kind, r.metadata.name])
}

# Pods should have an unprivileged security context set
resource_container_security_context_privileged[reason] {
    r := get_containers[_]
    containers := r.spec.containers[_]
    containers.securityContext["privileged"] == true
    reason := sprintf("%s %s: Container %s securityContext.privileged should not be set to 'true'.", [r.kind, r.metadata.name, containers.name])
}

deny_resource_container_security_context_privileged[msg]{
  not array_contains(warning_mode, "resource_container_security_context_privileged")
  resource_container_security_context_privileged[_] != []
  msg := resource_container_security_context_privileged[_]
}

warn_resource_container_security_context_privileged[msg]{
  array_contains(warning_mode, "resource_container_security_context_privileged")
  resource_container_security_context_privileged[_] != []
  msg := resource_container_security_context_privileged[_]
}