package kubernetes

deployments[r] {
    r := input
    input.kind == "Deployment"
}

# Deployments should have readiness probes configured
deployment_missing_readiness_probe[reason] {
    r := deployments[_]
    containers := r.spec.template.spec.containers[_]
    not containers.readinessProbe
    reason := sprintf("%s %s: Container %s is missing a Readiness Probe configuration. Readiness probes will tell you when your app is ready to serve traffic.", [r.kind, r.metadata.name, containers.name])
}

deny_deployment_missing_readiness_probe[msg]{
  not array_contains(warning_mode, "deployment_missing_readiness_probe")
  deployment_missing_readiness_probe[_] != []
  msg := deployment_missing_readiness_probe[_]
}

warn_deployment_missing_readiness_probe[msg]{
  array_contains(warning_mode, "deployment_missing_readiness_probe")
  deployment_missing_readiness_probe[_] != []
  msg := deployment_missing_readiness_probe[_]
}

# Deployments should have liveness probes configured
deployment_missing_liveness_probe[reason] {
    r := deployments[_]
    containers := r.spec.template.spec.containers[_]
    not containers.livenessProbe
    reason := sprintf("%s %s: Container %s is missing a Liveness Probe configuration. Liveness probes will let Kubernetes know whether your app is healthy.", [r.kind, r.metadata.name, containers.name])
}

deny_deployment_missing_liveness_probe[msg]{
  not array_contains(warning_mode, "deployment_missing_liveness_probe")
  deployment_missing_liveness_probe[_] != []
  msg := deployment_missing_liveness_probe[_]
}

warn_deployment_missing_liveness_probe[msg]{
  array_contains(warning_mode, "deployment_missing_liveness_probe")
  deployment_missing_liveness_probe[_] != []
  msg := deployment_missing_liveness_probe[_]
}

# Deployments selector labels match the template metadata labels
# NOTE: This is p3 and will only run in warning mode
warn_deployment_selector_labels_mismatched[reason] {
    r := deployments[_]
    matchLabels := { [label, value] | some label; value := r.spec.selector.matchLabels[label] }
    labels := { [label, value] | some label; value := r.spec.template.metadata.labels[label] }
    p1 := count(matchLabels - labels)
    p2 := count(labels - matchLabels)
    not p1 + p2 == 0
    reason := sprintf("[Recommended] %s %s: Deployment has mismatched selector labels. Make sure that spec.selector.matchLabels are the same as spec.template.metadata.labels", [r.kind, r.metadata.name])
}