package kubernetes

deprecated_k8s_apis := {
  # v1.27
  "storage.k8s.io/v1beta1": [
    "CSIStorageCapacity",
    # v1.22
    "CSIDriver",
    "CSINode",
    "StorageClass",
    "VolumeAttachment"
  ],
  # v1.26
  "flowcontrol.apiserver.k8s.io/v1beta1": [
    "FlowSchema",
    "PriorityLevelConfiguration"
  ],
  "autoscaling/v2beta2": [
    "HorizontalPodAutoscaler"
  ],
  # v1.25
  "batch/v1beta1": [
    "CronJob"
  ],
  "discovery.k8s.io/v1beta1": [
    "EndpointSlice"
  ],
  "events.k8s.io/v1beta1": [
    "Event"
  ],
  "autoscaling/v2beta1": [
    "HorizontalPodAutoscaler"
  ],
  "policy/v1beta1": [
    "PodDisruptionBudget",
    "PodSecurityPolicy"
  ],
  "node.k8s.io/v1beta1": [
    "RuntimeClass"
  ],
  # v1.22
  "admissionregistration.k8s.io/v1beta1": [
    "MutatingWebhookConfiguration",
    "ValidatingWebhookConfiguration"
  ],
  "apiextensions.k8s.io/v1beta1": [
    "CustomResourceDefinition"
  ],
  "apiregistration.k8s.io/v1beta1": [
    "APIService"
  ],
  "authentication.k8s.io/v1beta1": [
    "TokenReview"
  ],
  "authorization.k8s.io/v1beta1": [
    "LocalSubjectAccessReview",
    "SelfSubjectAccessReview",
    "SubjectAccessReview",
    "SelfSubjectRulesReview"
  ],
  "certificates.k8s.io/v1beta1": [
    "CertificateSigningRequest"
  ],
  "coordination.k8s.io/v1beta1": [
    "Lease"
  ],
  "extensions/v1beta1": [
    "Ingress",
    # v1.16
    "NetworkPolicy",
    "DaemonSet",
    "Deployment",
    "ReplicaSet",
    "PodSecurityPolicy"
  ],
  "networking.k8s.io/v1beta1": [
    "Ingress",
    "IngressClass"
  ],
  "rbac.authorization.k8s.io/v1beta1": [
    "ClusterRole",
    "ClusterRoleBinding",
    "Role",
    "RoleBinding"
  ],
  "scheduling.k8s.io/v1beta1": [
    "PriorityClass"
  ],
  # v1.16
  "apps/v1beta1": [
    "Deployment",
    "StatefulSet",
    "ReplicaSet"
  ],
  "apps/v1beta2": [
    "DaemonSet",
    "Deployment",
    "StatefulSet",
    "ReplicaSet"
  ]
}


# input.apiVersion
# NOTE: These are p3 and will only run in warning mode
resource_deprecated_api[msg] {
  deprecated_k8s_apis[input.apiVersion][_] == input.kind
  msg = sprintf("[Recommended] %s is using a deprecated API %s in apiVersion %s. Please refer to https://kubernetes.io/docs/reference/using-api/deprecation-guide/ for remediation guidance.", [name, input.kind, input.apiVersion])
}

warn_resource_deprecated_api[msg] {
  array_contains(warning_mode, "resource_deprecated_api")
  resource_deprecated_api[_] != []
  msg := resource_deprecated_api[_]
}

deny_resource_deprecated_api[msg] {
  not array_contains(warning_mode, "resource_deprecated_api")
  resource_deprecated_api[_] != []
  msg := resource_deprecated_api[_]
}
