package mosaicv2

################################################################################
##                             Changes Tests                                  ##
################################################################################

# Test for Recreate Actions
# State is empty, plan is only creating new resources
test_recreation_plan_all_new {
    i := data.mock.all_additions
    warned_recreate := warn_resource_recreation with input as i
    denied_recreate := deny_planned_recreations with input as i
    total_recreate = count(warned_recreate) + count(denied_recreate)
    total_recreate == 0
}

# Test for Recreate Actions
# State has resources, plan is deleting but not recreating resources
test_recreation_plan_all_deletions {
    i := data.mock.all_deletions
    warned_recreate := warn_resource_recreation with input as i
    denied_recreate := deny_planned_recreations with input as i
    total_recreate = count(warned_recreate) + count(denied_recreate)
    total_recreate == 0
}

# Test for Delete Actions
# State is empty, plan is only creating new resources
test_deletion_plan_all_new {
    i := data.mock.all_additions
    warned_deletion := warn_resource_deletion with input as i
    denied_deletion := deny_planned_deletions with input as i
    total_deletion = count(warned_deletion) + count(denied_deletion)
    total_deletion == 0
}

# Test for Delete Actions
# State has resources, plan is deleting resources
test_deletion_plan_all_deletions {
    i := data.mock.all_deletions
    warned_deletion := warn_resource_deletion with input as i # 3 warnings
    count(warned_deletion) == 3
    denied_deletion := deny_planned_deletions with input as i # 1 denial
    count(denied_deletion) == 1
    total_deletion = count(warned_deletion) + count(denied_deletion)
    total_deletion == 4
}

# Test for Recreate Actions
# State has resources, plan recreating resources
test_recreation_plan_mixed_actions {
    i := data.mock.blastradius_recreate
    warned_recreate := warn_resource_recreation with input as i # 2 warnings
    count(warned_recreate) == 1
    denied_recreate := deny_planned_recreations with input as i # 1 denial
    count(denied_recreate) == 1
    total_recreate = count(warned_recreate) + count(denied_recreate)
    total_recreate == 2
}

# Test for Any Actions
# State has resources, plan recreating AND deleting resources
test_plan_with_mixed_actions {
    i := data.mock.blastradius_recreate
    warned_recreate := warn_resource_recreation with input as i # 1 warnings
    denied_recreate := deny_planned_recreations with input as i # 1 denial
    total_recreate = count(warned_recreate) + count(denied_recreate)
    total_recreate == 2

    warned_deletion := warn_resource_deletion with input as i # 1 warnings
    denied_deletion := deny_planned_deletions with input as i # 1 denial
    total_deletion = count(warned_deletion) + count(denied_deletion)
    total_deletion == 0
}

################################################################################
##                           Prohibited Tests                                 ##
################################################################################

# Plan has a Cloudfront Key
test_prohibited_resource {
    i := data.mock.prohibited_cloudfront_key
    bucket_acl_result := warn_prohibited_resource with input as i
    count(bucket_acl_result) == 1
}

# Plan has a Cloudfront Key and Keygroup
test_prohibited_resources {
    i := data.mock.prohibited_resources
    bucket_acl_result := warn_prohibited_resource with input as i
    count(bucket_acl_result) == 2
}

# Plan has a Cloudfront Key and Keygroup, but also s3 buckets which are allowed
test_mixed_resources {
    i := data.mock.mixed_resources
    bucket_acl_result := warn_prohibited_resource with input as i
    count(bucket_acl_result) == 2
}

# Plan has a Cloudfront Key and Keygroup, but also s3 buckets which are allowed
# The Denylist has been updated to prohibit s3 but not cloudfront resources
test_new_prohibited_resource {
    i := data.mock.mixed_resources
    bucket_acl_result := warn_prohibited_resource with input as i with denylist as ["aws_s3_bucket"]
    count(bucket_acl_result) == 1
}

# Plan only has allowed resources
test_new_prohibited_resource {
    i := data.mock.allowed_resources
    bucket_acl_result := warn_prohibited_resource with input as i
    count(bucket_acl_result) == 0
}