package main

# Test s3 key check function
test_s3_tags_contain_proper_keys {
  tags := {
    "data_classification":	"private"
  }
  missing_tags := {}
  s3_tags_contain_proper_keys(tags)
  s3_tags_contain_proper_keys(tags)
  not s3_tags_contain_proper_keys(missing_tags)
}

# Test S3 ACL policy rules
test_bucket_acl_invalid {
    i := data.mock.invalid_s3_bucket_acls_input
    r1 := warn_bucket_acl with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_bucket_acl with input as i with data.config.warn_mode as ["bucket_acl"]
    count(r2) == 2
    r3 := deny_bucket_acl with input as i with data.config.warn_mode as []
    count(r3) == 2
    r4 := deny_bucket_acl with input as i with data.config.warn_mode as ["bucket_acl"]
    count(r4) == 0
    count(r1) + count(r2) == 2 # Warning mode is off
    count(r3) + count(r4) == 2 # Warning mode is on
}

test_bucket_acl_valid {
    i := data.mock.valid_s3_bucket_input
    r1 := warn_bucket_acl with input as i with data.config.warn_mode as []
    r2 := warn_bucket_acl with input as i with data.config.warn_mode as ["bucket_acl"]
    r3 := deny_bucket_acl with input as i with data.config.warn_mode as []
    r4 := deny_bucket_acl with input as i with data.config.warn_mode as ["bucket_acl"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}

# Test for invalid encryption parameters
test_bucket_server_side_encryption_invalid {
    i := data.mock.invalid_s3_bucket_encryption_input
    r1 := warn_bucket_server_side_encryption with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_bucket_server_side_encryption with input as i with data.config.warn_mode as ["bucket_server_side_encryption"]
    count(r2) == 1
    r3 := deny_bucket_server_side_encryption with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_bucket_server_side_encryption with input as i with data.config.warn_mode as ["bucket_server_side_encryption"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

# Test for invalid SSE algorithms
test_bucket_sse_algorithm_invalid {
    i := data.mock.invalid_s3_bucket_encryption_input
    r1 := warn_bucket_sse_algorithm with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_bucket_sse_algorithm with input as i with data.config.warn_mode as ["bucket_sse_algorithm"]
    count(r2) == 1
    r3 := deny_bucket_sse_algorithm with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_bucket_sse_algorithm with input as i with data.config.warn_mode as ["bucket_sse_algorithm"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

## this tests cases where versioning key is missing and explicit deny of versioning { enabled = false}
test_buckets_without_versioning_invalid {
    i := data.mock.invalid_s3_bucket_input_without_versioning
    r1 := warn_buckets_without_versioning with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_buckets_without_versioning with input as i with data.config.warn_mode as ["buckets_without_versioning"]
    count(r2) == 2
    r3 := deny_buckets_without_versioning with input as i with data.config.warn_mode as []
    count(r3) == 2
    r4 := deny_buckets_without_versioning with input as i with data.config.warn_mode as ["buckets_without_versioning"]
    count(r4) == 0
    count(r1) + count(r2) == 2 # Warning mode is off
    count(r3) + count(r4) == 2 # Warning mode is on
}

test_buckets_without_versioning_valid {
    i := data.mock.valid_s3_bucket_input_with_versioning_enabled
    r1 := warn_buckets_without_versioning with input as i with data.config.warn_mode as []
    r2 := warn_buckets_without_versioning with input as i with data.config.warn_mode as ["buckets_without_versioning"]
    r3 := deny_buckets_without_versioning with input as i with data.config.warn_mode as []
    r4 := deny_buckets_without_versioning with input as i with data.config.warn_mode as ["buckets_without_versioning"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}

# ## this test case is for when server access logging is enabled
# test_bucket_logging_invalid {
#     i := data.mock.invalid_s3_bucket_logging
#     r1 := warn_bucket_logging with input as i with data.config.warn_mode as []
#     count(r1) == 0
#     r2 := warn_bucket_logging with input as i with data.config.warn_mode as ["bucket_logging"]
#     count(r2) == 1
#     r3 := deny_bucket_logging with input as i with data.config.warn_mode as []
#     count(r3) == 1
#     r4 := deny_bucket_logging with input as i with data.config.warn_mode as ["bucket_logging"]
#     count(r4) == 0
#     count(r1) + count(r2) == 1 # Warning mode is off
#     count(r3) + count(r4) == 1 # Warning mode is on
# }

# test_bucket_logging_valid {
#     i := data.mock.valid_s3_bucket_logging
#     r1 := warn_bucket_logging with input as i with data.config.warn_mode as []
#     r2 := warn_bucket_logging with input as i with data.config.warn_mode as ["bucket_logging"]
#     r3 := deny_bucket_logging with input as i with data.config.warn_mode as []
#     r4 := deny_bucket_logging with input as i with data.config.warn_mode as ["bucket_logging"]
#     count(r1) + count(r2) + count(r3) + count(r4) == 0
# }

## this test case is for when server access logging is disabled
test_bucket_log_retention_invalid {
    i := data.mock.s3_bucket_retention
    r1 := warn_bucket_log_retention with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_bucket_log_retention with input as i with data.config.warn_mode as ["bucket_log_retention"]
    count(r2) == 1
    r3 := deny_bucket_log_retention with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_bucket_log_retention with input as i with data.config.warn_mode as ["bucket_log_retention"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

# Test for missing required s3 tags
test_missing_s3_tags_invalid {
    i := data.mock.invalid_s3_bucket_tags_input
    r1 := warn_missing_s3_tags with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_missing_s3_tags with input as i with data.config.warn_mode as ["missing_s3_tags"]
    count(r2) == 1
    r3 := deny_missing_s3_tags with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_missing_s3_tags with input as i with data.config.warn_mode as ["missing_s3_tags"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

test_missing_s3_tags_valid {
    i := data.mock.valid_s3_bucket_tags_input
    r1 := warn_missing_s3_tags with input as i with data.config.warn_mode as []
    r2 := warn_missing_s3_tags with input as i with data.config.warn_mode as ["missing_s3_tags"]
    r3 := deny_missing_s3_tags with input as i with data.config.warn_mode as []
    r4 := deny_missing_s3_tags with input as i with data.config.warn_mode as ["missing_s3_tags"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}
