package main
import data.config

minimum_tags = {"owner","sguid","src_url","stage","sku"}
resource_exclusions = config.tag_validation.resource_exclusions

# Get Terrform Plan Changeset
module_address[i] = address {
	changeset := input.resource_changes[i]
	address := changeset.address
}

################################################################################
## Test Cases                                                                 ##
################################################################################

# r.change.after.tags[_].key case
# Normal tagging for AWS resource, the terraform plan stores the values under `*.tags[]`
resource_missing_tags[msg] {
    r := input.resource_changes[_]      # Get resource changes
    not r.mode == "data"                # Ignore data resources when checking tags
    is_array(r.change.after.tags)       # Check that 'tags' is an array
    r.change.after.tag == []            # Check that 'tag' is an empty array
    keys := {keys | keys = r.change.after.tags[_].key}  # Parse tags for keys
    leftover := minimum_tags - keys     # Diff between found keys and expected keys
    not leftover == set()               # If there are remaining keys, fail
    msg := sprintf("Missing required tags %v for the following resource: [%v]. \nPlease see SBU tagging policy for more information https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy", [leftover, r.address])
}

# r.change.after.tags[_].keys case
# When using dynamic tags on AWS resources, the terraform plan stores the values under `*.tag[]`
resource_missing_tags[msg] {
    r := input.resource_changes[_]
    not r.mode == "data"                # Ignore data resources when checking tags
    is_array(r.change.after.tag)
    r.change.after.tags == []
    keys := {keys | keys = r.change.after.tag[_].key}
    leftover := minimum_tags - keys
    not leftover == set()
    msg := sprintf("Missing required tags %v for the following resource: [%v]. \nPlease see SBU tagging policy for more information https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy [Dynamic Tags]", [leftover, r.address])
}

deny_resource_missing_tags[msg]{
    not array_contains(warning_mode, "resource_missing_tags")
    resource_missing_tags[_] != []
    msg := resource_missing_tags[_]
}

warn_resource_missing_tags[msg]{
    array_contains(warning_mode, "resource_missing_tags")
    resource_missing_tags[_] != []
    msg := resource_missing_tags[_]
}

# === Bad Owner Tag ===
# Check tags for owner tag, and that owner tag has a valid email
contains_valid_email(key, val) {
	key == "owner"
	re_match(`^([\.a-zA-Z0-9+_-]+)(@vmware\.com|@(.*).vmware\.com)$`, val)
} else {
	key != "owner"
}

owner_tag_valid[i] = resources {
	changeset := input.resource_changes[i]
	tags := changeset.change.after.tags
	resources := [resource |
        # Ignore excluded resources from config
        not array_contains(resource_exclusions, changeset.type);
        # Ignore data resources when checking tags
        not changeset.mode == "data";
        # Check owner tag is valid email
        val := tags[key];
        not contains_valid_email(key, val);
        resource := module_address[i]
	]
}

# Check AWS resource 'owner' tag for a valid @vmare.com email address
bad_owner_tag[msg] {
	resources := owner_tag_valid[_]
	resources != []
	msg := sprintf("Tag ['owner'] does not contain VMware email address for the following resources: %v. \nPlease see SBU tagging policy for more information https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy", [resources])
}

deny_bad_owner_tag[msg]{
  not array_contains(warning_mode, "bad_owner_tag")
  bad_owner_tag[_] != []
  msg := bad_owner_tag[_]
}

warn_bad_owner_tag[msg]{
  array_contains(warning_mode, "bad_owner_tag")
  bad_owner_tag[_] != []
  msg := bad_owner_tag[_]
}

# === Bad sguid tag ===
# Check tags for owner tag, and that owner tag has a valid email
contains_valid_sguid_value(key, val) {
	key == "sguid"
	not contains( lower(val), `dummy`)
	not contains( lower(val), `none`)
} else {
	key != "sguid"
}

sguid_valid_value[i] = resources {
	changeset := input.resource_changes[i]
	tags := changeset.change.after.tags
	resources := [resource |
        # Ignore excluded resources from config
        not array_contains(resource_exclusions, changeset.type);
        # Ignore data resources when checking tags
        not changeset.mode == "data";
        # Check owner tag is valid email
        val := tags[key];
        not contains_valid_sguid_value(key, val);
        resource := module_address[i]
	]
}

# Check AWS resource 'sguid' tag for invalid value
bad_sguid_tag[msg] {
	resources := sguid_valid_value[_]
	resources != []
	msg := sprintf("Tag ['sguid'] does not contain valid value for the following resources: %v. sguid tag should not have value Dummy or NONE. \nPlease see SBU tagging policy for more information https://confluence.carbonblack.local/display/cbengx/SBU+AWS+Tagging+Policy", [resources])
}

deny_bad_sguid_tag[msg]{
  not array_contains(warning_mode, "bad_sguid_tag")
  bad_sguid_tag[_] != []
  msg := bad_sguid_tag[_]
}

warn_bad_sguid_tag[msg]{
  array_contains(warning_mode, "bad_sguid_tag")
  bad_sguid_tag[_] != []
  msg := bad_sguid_tag[_]
}
