package main

# Test valid email function
test_contains_valid_email {
    resource_exclusions := {"aws_iam_policy"}
    not contains_valid_email("owner", "cb-prod-infra-teamcarbonblack.com")
    contains_valid_email("owner", "cb-prod-infra-team@vmware.com")
    contains_valid_email("Name", "cb-prod-infra-resource")
    contains_valid_email("Name", "cb-prod-infra-team@vmware.com")
    contains_valid_email("owner", "cb-prod-infra-team@vmware.com")
    contains_valid_email("Name", "cb-prod-infra-resource")
    resource_exclusions
    not resource_exclusions == {""}
    resource_exclusions == {"aws_iam_policy"}
}

# Test to check valid sguid value
test_contains_valid_sguid {
    resource_exclusions := {"aws_iam_policy"}
    contains_valid_sguid_value("sguid", "prodeng:aegis:aws:ec2")
    not contains_valid_sguid_value("sguid", "prodeng:DUMMY")
    not contains_valid_sguid_value("sguid", "defense:dev:infra:dummy")
    not contains_valid_sguid_value("sguid", "none:none:none")
    not contains_valid_sguid_value("sguid", "dummy:dummy:dymmy:ec2")
    not contains_valid_sguid_value("sguid", "prodeng:aegis:aws:none")
    contains_valid_sguid_value("owner", "prodeng:aegis:aws:dummy")
    contains_valid_sguid_value("Name", "prodeng:aegis:aws:s3")
    resource_exclusions
    not resource_exclusions == {""}
    resource_exclusions == {"aws_iam_policy"}
}

# Test for invalid owner tag
test_bad_sguid_tag_invalid {
    i := data.mock.invalid_tags_input
    r1 := warn_bad_sguid_tag with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_bad_sguid_tag with input as i with data.config.warn_mode as ["bad_sguid_tag"]
    count(r2) == 1
    r3 := deny_bad_sguid_tag with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_bad_sguid_tag with input as i with data.config.warn_mode as ["bad_sguid_tag"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

# Test for invalid owner tag
test_bad_sguid_tag_valid {
    i := data.mock.valid_tags_input
    r1 := warn_bad_sguid_tag with input as i with data.config.warn_mode as []
    r2 := warn_bad_sguid_tag with input as i with data.config.warn_mode as ["bad_sguid_tag"]
    r3 := deny_bad_sguid_tag with input as i with data.config.warn_mode as []
    r4 := deny_bad_sguid_tag with input as i with data.config.warn_mode as ["bad_sguid_tag"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0

}

# Test for invalid owner tag
test_bad_owner_tag_invalid {
    i := data.mock.invalid_tags_input
    r1 := warn_bad_owner_tag with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_bad_owner_tag with input as i with data.config.warn_mode as ["bad_owner_tag"]
    count(r2) == 1
    r3 := deny_bad_owner_tag with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_bad_owner_tag with input as i with data.config.warn_mode as ["bad_owner_tag"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

# Test for invalid owner tag
test_resource_missing_tags {
    i := data.mock.asg_tags_input
    r1 := warn_resource_missing_tags with input as i with data.config.warn_mode as []
    r2 := warn_resource_missing_tags with input as i with data.config.warn_mode as ["resource_missing_tags"]
    r3 := deny_resource_missing_tags with input as i with data.config.warn_mode as []
    r4 := deny_resource_missing_tags with input as i with data.config.warn_mode as ["resource_missing_tags"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}

test_resource_missing_tags_invalid {
    i := data.mock.invalid_asg_tags_input
    r1 := warn_resource_missing_tags with input as i with data.config.warn_mode as []
    count(r1) == 0
    r2 := warn_resource_missing_tags with input as i with data.config.warn_mode as ["resource_missing_tags"]
    count(r2) == 1
    r3 := deny_resource_missing_tags with input as i with data.config.warn_mode as []
    count(r3) == 1
    r4 := deny_resource_missing_tags with input as i with data.config.warn_mode as ["resource_missing_tags"]
    count(r4) == 0
    count(r1) + count(r2) == 1 # Warning mode is off
    count(r3) + count(r4) == 1 # Warning mode is on
}

# Test for invalid owner tag
test_resource_missing_tags_dynamic {
    i := data.mock.asg_dynamic_tags
    r1 := warn_resource_missing_tags with input as i with data.config.warn_mode as []
    r2 := warn_resource_missing_tags with input as i with data.config.warn_mode as ["resource_missing_tags"]
    r3 := deny_resource_missing_tags with input as i with data.config.warn_mode as []
    r4 := deny_resource_missing_tags with input as i with data.config.warn_mode as ["resource_missing_tags"]
    count(r1) + count(r2) + count(r3) + count(r4) == 0
}
